@@ -19,17 +19,18 @@
 /**
  * Generate a 1x1 <code>Picture</code> of a particular color.
  */
-xcb_render_picture_t solid_picture(xcb_connection_t *c, xcb_drawable_t d, bool argb,
-                                   double a, double r, double g, double b) {
+xcb_render_picture_t
+solid_picture(struct x_connection *c, bool argb, double a, double r, double g, double b) {
 	xcb_pixmap_t pixmap;
 	xcb_render_picture_t picture;
 	xcb_render_create_picture_value_list_t pa;
 	xcb_render_color_t col;
 	xcb_rectangle_t rect;
 
-	pixmap = x_create_pixmap(c, argb ? 32 : 8, d, 1, 1);
-	if (!pixmap)
+	pixmap = x_create_pixmap(c, argb ? 32 : 8, 1, 1);
+	if (!pixmap) {
 		return XCB_NONE;
+	}
 
 	pa.repeat = 1;
 	picture = x_create_picture_with_standard_and_pixmap(
@@ -37,7 +38,7 @@ xcb_render_picture_t solid_picture(xcb_connection_t *c, xcb_drawable_t d, bool a
 	    XCB_RENDER_CP_REPEAT, &pa);
 
 	if (!picture) {
-		xcb_free_pixmap(c, pixmap);
+		xcb_free_pixmap(c->c, pixmap);
 		return XCB_NONE;
 	}
 
@@ -51,14 +52,14 @@ xcb_render_picture_t solid_picture(xcb_connection_t *c, xcb_drawable_t d, bool a
 	rect.width = 1;
 	rect.height = 1;
 
-	xcb_render_fill_rectangles(c, XCB_RENDER_PICT_OP_SRC, picture, col, 1, &rect);
-	xcb_free_pixmap(c, pixmap);
+	xcb_render_fill_rectangles(c->c, XCB_RENDER_PICT_OP_SRC, picture, col, 1, &rect);
+	xcb_free_pixmap(c->c, pixmap);
 
 	return picture;
 }
 
-xcb_image_t *
-make_shadow(xcb_connection_t *c, const conv *kernel, double opacity, int width, int height) {
+xcb_image_t *make_shadow(struct x_connection *c, const conv *kernel, double opacity,
+                         int width, int height) {
 	/*
 	 * We classify shadows into 4 kinds of regions
 	 *    r = shadow radius
@@ -84,8 +85,9 @@ make_shadow(xcb_connection_t *c, const conv *kernel, double opacity, int width,
 	assert(d % 2 == 1);
 	assert(d > 0);
 
-	ximage = xcb_image_create_native(c, to_u16_checked(swidth), to_u16_checked(sheight),
-	                                 XCB_IMAGE_FORMAT_Z_PIXMAP, 8, 0, 0, NULL);
+	ximage =
+	    xcb_image_create_native(c->c, to_u16_checked(swidth), to_u16_checked(sheight),
+	                            XCB_IMAGE_FORMAT_Z_PIXMAP, 8, 0, 0, NULL);
 	if (!ximage) {
 		log_error("failed to create an X image");
 		return 0;
@@ -193,7 +195,7 @@ make_shadow(xcb_connection_t *c, const conv *kernel, double opacity, int width,
 /**
  * Generate shadow <code>Picture</code> for a window.
  */
-bool build_shadow(xcb_connection_t *c, xcb_drawable_t d, double opacity, const int width,
+bool build_shadow(struct x_connection *c, double opacity, const int width,
                   const int height, const conv *kernel, xcb_render_picture_t shadow_pixel,
                   xcb_pixmap_t *pixmap, xcb_render_picture_t *pict) {
 	xcb_image_t *shadow_image = NULL;
@@ -207,9 +209,9 @@ bool build_shadow(xcb_connection_t *c, xcb_drawable_t d, double opacity, const i
 		return false;
 	}
 
-	shadow_pixmap = x_create_pixmap(c, 8, d, shadow_image->width, shadow_image->height);
+	shadow_pixmap = x_create_pixmap(c, 8, shadow_image->width, shadow_image->height);
 	shadow_pixmap_argb =
-	    x_create_pixmap(c, 32, d, shadow_image->width, shadow_image->height);
+	    x_create_pixmap(c, 32, shadow_image->width, shadow_image->height);
 
 	if (!shadow_pixmap || !shadow_pixmap_argb) {
 		log_error("Failed to create shadow pixmaps");
@@ -225,11 +227,11 @@ bool build_shadow(xcb_connection_t *c, xcb_drawable_t d, double opacity, const i
 	}
 
 	gc = x_new_id(c);
-	xcb_create_gc(c, gc, shadow_pixmap, 0, NULL);
+	xcb_create_gc(c->c, gc, shadow_pixmap, 0, NULL);
 
 	// We need to make room for protocol metadata in the request. The metadata should
 	// be 24 bytes plus padding, let's be generous and give it 1kb
-	auto maximum_image_size = xcb_get_maximum_request_length(c) * 4 - 1024;
+	auto maximum_image_size = xcb_get_maximum_request_length(c->c) * 4 - 1024;
 	auto maximum_row =
 	    to_u16_checked(clamp(maximum_image_size / shadow_image->stride, 0, UINT16_MAX));
 	if (maximum_row <= 0) {
@@ -248,23 +250,23 @@ bool build_shadow(xcb_connection_t *c, xcb_drawable_t d, double opacity, const i
 		}
 
 		uint32_t offset = row * shadow_image->stride / sizeof(*shadow_image->data);
-		xcb_put_image(c, (uint8_t)shadow_image->format, shadow_pixmap, gc,
+		xcb_put_image(c->c, (uint8_t)shadow_image->format, shadow_pixmap, gc,
 		              shadow_image->width, batch_height, 0, to_i16_checked(row),
 		              0, shadow_image->depth, shadow_image->stride * batch_height,
 		              shadow_image->data + offset);
 	}
 
-	xcb_render_composite(c, XCB_RENDER_PICT_OP_SRC, shadow_pixel, shadow_picture,
+	xcb_render_composite(c->c, XCB_RENDER_PICT_OP_SRC, shadow_pixel, shadow_picture,
 	                     shadow_picture_argb, 0, 0, 0, 0, 0, 0, shadow_image->width,
 	                     shadow_image->height);
 
 	*pixmap = shadow_pixmap_argb;
 	*pict = shadow_picture_argb;
 
-	xcb_free_gc(c, gc);
+	xcb_free_gc(c->c, gc);
 	xcb_image_destroy(shadow_image);
-	xcb_free_pixmap(c, shadow_pixmap);
-	xcb_render_free_picture(c, shadow_picture);
+	xcb_free_pixmap(c->c, shadow_pixmap);
+	x_free_picture(c, shadow_picture);
 
 	return true;
 
@@ -273,41 +275,43 @@ shadow_picture_err:
 		xcb_image_destroy(shadow_image);
 	}
 	if (shadow_pixmap) {
-		xcb_free_pixmap(c, shadow_pixmap);
+		xcb_free_pixmap(c->c, shadow_pixmap);
 	}
 	if (shadow_pixmap_argb) {
-		xcb_free_pixmap(c, shadow_pixmap_argb);
+		xcb_free_pixmap(c->c, shadow_pixmap_argb);
 	}
 	if (shadow_picture) {
-		xcb_render_free_picture(c, shadow_picture);
+		x_free_picture(c, shadow_picture);
 	}
 	if (shadow_picture_argb) {
-		xcb_render_free_picture(c, shadow_picture_argb);
+		x_free_picture(c, shadow_picture_argb);
 	}
 	if (gc) {
-		xcb_free_gc(c, gc);
+		xcb_free_gc(c->c, gc);
 	}
 
 	return false;
 }
 
-void *default_backend_render_shadow(backend_t *backend_data, int width, int height,
-                                    struct backend_shadow_context *sctx, struct color color) {
+void *default_render_shadow(backend_t *backend_data, int width, int height,
+                            struct backend_shadow_context *sctx, struct color color) {
 	const conv *kernel = (void *)sctx;
-	xcb_pixmap_t shadow_pixel = solid_picture(backend_data->c, backend_data->root, true,
-	                                          1, color.red, color.green, color.blue),
-	             shadow = XCB_NONE;
+	xcb_render_picture_t shadow_pixel =
+	    solid_picture(backend_data->c, true, 1, color.red, color.green, color.blue);
+	xcb_pixmap_t shadow = XCB_NONE;
 	xcb_render_picture_t pict = XCB_NONE;
 
-	if (!build_shadow(backend_data->c, backend_data->root, color.alpha, width, height,
-	                  kernel, shadow_pixel, &shadow, &pict)) {
+	if (!build_shadow(backend_data->c, color.alpha, width, height, kernel,
+	                  shadow_pixel, &shadow, &pict)) {
+		x_free_picture(backend_data->c, shadow_pixel);
 		return NULL;
 	}
 
 	auto visual = x_get_visual_for_standard(backend_data->c, XCB_PICT_STANDARD_ARGB_32);
 	void *ret = backend_data->ops->bind_pixmap(
 	    backend_data, shadow, x_get_visual_info(backend_data->c, visual), true);
-	xcb_render_free_picture(backend_data->c, pict);
+	x_free_picture(backend_data->c, pict);
+	x_free_picture(backend_data->c, shadow_pixel);
 	return ret;
 }
 
@@ -401,31 +405,31 @@ struct dual_kawase_params *generate_dual_kawase_params(void *args) {
 		int min_radius;        /// Approximate gauss-blur with at least this
 		                       /// radius and std-deviation
 	} strength_levels[20] = {
-	    {.iterations = 1, .offset = 1.25f, .min_radius = 1},          // LVL  1
-	    {.iterations = 1, .offset = 2.25f, .min_radius = 6},          // LVL  2
-	    {.iterations = 2, .offset = 2.00f, .min_radius = 11},         // LVL  3
-	    {.iterations = 2, .offset = 3.00f, .min_radius = 17},         // LVL  4
-	    {.iterations = 2, .offset = 4.25f, .min_radius = 24},         // LVL  5
-	    {.iterations = 3, .offset = 2.50f, .min_radius = 32},         // LVL  6
-	    {.iterations = 3, .offset = 3.25f, .min_radius = 40},         // LVL  7
-	    {.iterations = 3, .offset = 4.25f, .min_radius = 51},         // LVL  8
-	    {.iterations = 3, .offset = 5.50f, .min_radius = 67},         // LVL  9
-	    {.iterations = 4, .offset = 3.25f, .min_radius = 83},         // LVL 10
-	    {.iterations = 4, .offset = 4.00f, .min_radius = 101},        // LVL 11
-	    {.iterations = 4, .offset = 5.00f, .min_radius = 123},        // LVL 12
-	    {.iterations = 4, .offset = 6.00f, .min_radius = 148},        // LVL 13
-	    {.iterations = 4, .offset = 7.25f, .min_radius = 178},        // LVL 14
-	    {.iterations = 4, .offset = 8.25f, .min_radius = 208},        // LVL 15
-	    {.iterations = 5, .offset = 4.50f, .min_radius = 236},        // LVL 16
-	    {.iterations = 5, .offset = 5.25f, .min_radius = 269},        // LVL 17
-	    {.iterations = 5, .offset = 6.25f, .min_radius = 309},        // LVL 18
-	    {.iterations = 5, .offset = 7.25f, .min_radius = 357},        // LVL 19
-	    {.iterations = 5, .offset = 8.50f, .min_radius = 417},        // LVL 20
+	    {.iterations = 1, .offset = 1.25F, .min_radius = 1},          // LVL  1
+	    {.iterations = 1, .offset = 2.25F, .min_radius = 6},          // LVL  2
+	    {.iterations = 2, .offset = 2.00F, .min_radius = 11},         // LVL  3
+	    {.iterations = 2, .offset = 3.00F, .min_radius = 17},         // LVL  4
+	    {.iterations = 2, .offset = 4.25F, .min_radius = 24},         // LVL  5
+	    {.iterations = 3, .offset = 2.50F, .min_radius = 32},         // LVL  6
+	    {.iterations = 3, .offset = 3.25F, .min_radius = 40},         // LVL  7
+	    {.iterations = 3, .offset = 4.25F, .min_radius = 51},         // LVL  8
+	    {.iterations = 3, .offset = 5.50F, .min_radius = 67},         // LVL  9
+	    {.iterations = 4, .offset = 3.25F, .min_radius = 83},         // LVL 10
+	    {.iterations = 4, .offset = 4.00F, .min_radius = 101},        // LVL 11
+	    {.iterations = 4, .offset = 5.00F, .min_radius = 123},        // LVL 12
+	    {.iterations = 4, .offset = 6.00F, .min_radius = 148},        // LVL 13
+	    {.iterations = 4, .offset = 7.25F, .min_radius = 178},        // LVL 14
+	    {.iterations = 4, .offset = 8.25F, .min_radius = 208},        // LVL 15
+	    {.iterations = 5, .offset = 4.50F, .min_radius = 236},        // LVL 16
+	    {.iterations = 5, .offset = 5.25F, .min_radius = 269},        // LVL 17
+	    {.iterations = 5, .offset = 6.25F, .min_radius = 309},        // LVL 18
+	    {.iterations = 5, .offset = 7.25F, .min_radius = 357},        // LVL 19
+	    {.iterations = 5, .offset = 8.50F, .min_radius = 417},        // LVL 20
 	};
 
 	auto params = ccalloc(1, struct dual_kawase_params);
 	params->iterations = 0;
-	params->offset = 1.0f;
+	params->offset = 1.0F;
 
 	if (blur_args->strength <= 0 && blur_args->size) {
 		// find highest level that approximates blur-strength with the selected
@@ -449,7 +453,7 @@ struct dual_kawase_params *generate_dual_kawase_params(void *args) {
 	// - Smallest texture dimensions are halved `iterations`-times
 	// - Upsample needs pixels two-times `offset` away from the border
 	// - Plus one for interpolation differences
-	params->expand = (1 << params->iterations) * 2 * (int)ceil(params->offset) + 1;
+	params->expand = (1 << params->iterations) * 2 * (int)ceilf(params->offset) + 1;
 
 	return params;
 }
@@ -504,9 +508,8 @@ struct backend_image *default_new_backend_image(int w, int h) {
 }
 
 void init_backend_base(struct backend_base *base, session_t *ps) {
-	base->c = ps->c;
+	base->c = &ps->c;
 	base->loop = ps->loop;
-	base->root = ps->root;
 	base->busy = false;
 	base->ops = NULL;
 }
